//   Advanced Example:
//   This script is a demonstration of some nice tricks that can be performed in any script. For advanced
//   writers only! If you're a begginer writer, maybe you prefer to start with Banjo's tutorial script
//   available at SexScripts' webpage. If it looks too hard to start with for you, you may prefer the simpler
//   'simpleexample.groovy'. If what you're looking for is an example of the SexScripts API functions,
//   'test.groovy' is the example for you. If you're going with this advanced example, you better run it
//   first to see it in action, and then, have a look at the code. You can use any of the code here
//   in your script. And there's no need to give any credit!



// First the setInfos call. Look at how we define the background color: we load it from the SexScripts data,
// and if there is not a color under the key "advanced.bgcolor", we'll use 0x8000FF (lilac) as default

setInfos(4, "Advanced Example", "Example for fancy tricks to make attractive scripts. For advanced writers.",
		    "Wololo", "complete", loadInteger("advanced.bgcolor") ?: 0x8000FF, "en", ["test"])

// This works because null turns into false if evaluated as a boolean. But be careful! If the saved color under
// this key is black (0x000000) it will also be evaluated as false, and 0x8000FF will be used for background
// color instead. Any other color will be loaded without any issue. But if you are in the really need of loading
// black, use the color 0x000001 instead of 0x000000


// Speed of the appearing text when shown dynamically
def textSpeed = 40

// Fade in/out speed of pictures when shown dynamically
def imageSpeed = 40


// Now let's define some fancy closures

//	showDynamically(message)
//		Works like show(message), but the message is shown
//		gradually instead of in one shot
//		parameters:
//			message	 Message to be shown. It can't be
//					 null
//
def showDynamically = { message ->	
	if (textSpeed <= 0) textSpeed = 0.1
	def appearedMessage = ""

	for (c in message) {
		appearedMessage += c
		show(appearedMessage)
		wait(1/textSpeed)
	}
}


//	unshowDynamically(message)
//		This time the message will be gradually disappearing
//		parameters:
//			message	 Message to be disapeared. It can't be
//					 null
//
def unshowDynamically = { message ->	
	if (textSpeed <= 0) textSpeed = 0.1
	def i = message.size()

	while(i > 0) {
		show(message.substring(0, i--))
		wait(1/textSpeed)
	}
	
	show(null)
}


//	playSwats(swats, time)
//		Plays a number of swats
//		parameters:
//			swats	Number of swats to play
//			time		Seconds to wait between swats
//
def playSwats = { swats, time ->
	(1..swats).each {
		playBackgroundSound("swat.wav")
		wait(time)
	}
}


//	saveUserPicture(picture)
//		Ask the user for a picture and saves a copy of it in the 
//		images folder
//		parameters:
//			picture		Path where the copy of the picture is
//						going to be saved
//
//		returns:			true if everything gone ok, and false if
//						the user didn't select a picture
//
def saveUserPicture = { picture ->
	def filePath = getFile("Select a picture")
	if (filePath == null) return false

	new File("images/"+picture).delete()
	new File("images/"+picture) << new File(filePath).getBytes()
	return true
}


//	waitWithCount(time)
//		Waits for a number of seconds while showing the time count
//		parameters:
//			time		Seconds to wait
//
def waitWithCount = { time ->
	def startTime = getTime()
	def waitedTime = getTime()-startTime

	while ( waitedTime < time ) {
		int minutes = waitedTime / 60
		int seconds = waitedTime % 60

		def minutesExtraDigit = ""
		if (minutes < 10) minutesExtraDigit = "0"

		def secondsExtraDigit = ""
		if (seconds < 10) secondsExtraDigit = "0"

		show( minutesExtraDigit + minutes + ":" + secondsExtraDigit + seconds )
		waitedTime = getTime()-startTime
	}
}


//	waitWithCountDown(time)
//		Waits for a number of seconds while showing the time countdown
//		parameters:
//			time		Seconds to wait
//
def waitWithCountDown = { time ->
	def startTime = getTime()
	def waitedTime = getTime()-startTime

	while ( waitedTime < time ) {
		int minutes = (time - waitedTime) / 60
		int seconds = (time - waitedTime) % 60

		def minutesExtraDigit = ""
		if (minutes < 10) minutesExtraDigit = "0"

		def secondsExtraDigit = ""
		if (seconds < 10) secondsExtraDigit = "0"

		show( minutesExtraDigit + minutes + ":" + secondsExtraDigit + seconds )
		waitedTime = getTime()-startTime
	}
}


//	getRandomItem(itemKeys)
//		Randomly picks one key from a list of keys to the SexScripts data
//		that is storing true. Useful to randomly pick one toy or clothing
//		from a list
//		parameters:
//			itemKeys		List of boolean keys of SexScripts data
//		
//		returns:		The key of the randomly selected item or null if no
//					one is true
//
def getRandomItem = { itemKeys ->
	for ( key in itemKeys ) {
		if ( !loadBoolean(key) ) itemKeys -= key
	}

	if ( itemKeys.isEmpty() ) return null

	return itemKeys[ getRandom(itemKeys.size()) ]
}


//	setImageFancy(picture)
//		Works like setImage(picture), but the picture is faded in
//		instead of shown in one shot
//		parameters:
//			picture	 Picture to be shown. It can't be
//					 null
//
def setImageFancy = { picture ->
	if (imageSpeed <= 0) imageSpeed = 0.1
	def image = java.awt.Toolkit.getDefaultToolkit().getImage("images/"+picture)
	def width = image.getWidth()
	def height = image.getHeight()
	
	(1..10).each {
		def frameBuffer = new java.awt.image.BufferedImage( (int)((width*it)/10), (int)((height*it)/10),
 							  java.awt.image.BufferedImage.TYPE_4BYTE_ABGR)
		def sprite = javax.imageio.ImageIO.read(new File("images/"+picture))
	
		java.awt.Graphics2D graphics = frameBuffer.createGraphics()
	
		graphics.drawImage(sprite, 0, 0, (int)((width*it)/10), (int)((height*it)/10), null)
	
		setImage(frameBuffer, false)
		wait(1/imageSpeed)
	}
}


//	setImageOffFancy(picture)
//		Fades out a picture
//		parameters:
//			picture	 Picture to be faded out. It can't be
//					 null
//
def setImageOffFancy = { picture ->
	if (imageSpeed <= 0) imageSpeed = 0.1
	def image = java.awt.Toolkit.getDefaultToolkit().getImage("images/"+picture)
	def width = image.getWidth()
	def height = image.getHeight()
	
	(1..10).each {
		def frameBuffer = new java.awt.image.BufferedImage( (int)(width/it), (int)(height/it),
 							  java.awt.image.BufferedImage.TYPE_4BYTE_ABGR)
		def sprite = javax.imageio.ImageIO.read(new File("images/"+picture))
	
		java.awt.Graphics2D graphics = frameBuffer.createGraphics()
	
		graphics.drawImage(sprite, 0, 0, (int)(width/it), (int)(height/it), null)
	
		setImage(frameBuffer, false)
		wait(1/imageSpeed)
	}
	setImage(null)
}


// Here begins the 'real' execution of the code

// This variable tells us if the script has been just opened, or if we've returned from a
// background color change
def status = loadString("advanced.status") ?: "start"

if (status == "start") {
	// clean screen
	show(null)
	setImage(null)
	
	show("Welcome to this advanced example!")
	wait(4)

	show("This is a message shown with a simple show(message) call")
	wait(5.5)

	// Let's show some text dynamically
	showDynamically("But I am a cooler message because I'm appearing gradually.")
	wait(4.5)

	showDynamically("I can even appear at different speeds")
	wait(4.5)

	// We can change the speed the text appears at
	textSpeed = 8
	showDynamically("I am appearing slower now...")
	wait(4)

	textSpeed = 80
	showDynamically("And I am appearing much faster now!")
	wait(4.5)

	textSpeed = 40
	show(null)

	setImage("default-female.jpg")
	showDynamically("This is a picture shown with a simple setImage(picture) call")
	wait(4.5)

	setImage("default-male.jpg")
	showDynamically("I'm strong! but I'm bored of appearing always in one shot")
	wait(4.5)

	show(null)
	setImage(null)
	wait(2.5)

	// Let's fade in and fade out pictures
	setImageFancy("default-female.jpg")
	showDynamically("Hello!\n\nI'm the default female!")
	showButton("Great!")

	setImageOffFancy("default-female.jpg")
	setImageFancy("default-male.jpg")
	showDynamically("And I'm the default male.")
	wait(3.5)

	setImageOffFancy("default-male.jpg")
	setImageFancy("default-female.jpg")
	showDynamically("Isn't this way of appearing cooler?")
	wait(4)

	showDynamically("Let's play some swats in a easy way!")
	wait(4)
	playSwats(6, 1.2)
	wait(3)

	// Now let's see how easy is to save a picture from the user:

	showDynamically("And now send me any picture!")
	wait(4)

	if (saveUserPicture("userPicture.jpg")) {
		showDynamically("Wonderful!\n\nYou wanna see it?")
		wait(4)
		showButton("Yeah!")
		
		show(null)
		setImageOffFancy("default-female.jpg")
		wait(1.5)

		setImage("userPicture.jpg")
		wait(1.5)
		show("This is your picture")
		wait(4)
		
		show(null)
		wait(1)
		setImage(null)
		wait(1.5)
		setImageFancy("default-female.jpg")
		wait(1)
	}
	else {
		// This block executes if the user has closed the file choosing window
		// without choosing any picture
		showDynamically("You should have selected a picture...")
		wait(4)
	}

	showDynamically("Now let's change the background color!")
	wait(3)

	// To change the background color we restart the script with the new background color.
	// So first we save where we left so we can continue at this point after restarting
	save("advanced.status", "red")
	save("advanced.bgcolor", 0xFF0000)
	wait(1.5)

	return "advancedexample"
}

else if (status == "red") {
	// The bg color has been changed to red!
	// Let's first of all delete the saved data for if the script is closed it can start from
	// the begining when opened again
	save("advanced.status", null)
	save("advanced.bgcolor", null)
	wait(3)

	showDynamically("Too red!")
	wait(3)

	showDynamically("Let's change it to black!")
	wait(2.5)
	
	// Now let's put the bg color black, note we use 0x000001 instead of 0x000000
	save("advanced.status", "black")
	save("advanced.bgcolor", 0x000001)
	wait(1.5)

	return "advancedexample"
}

else if (status == "black") {
	// The bg color has been changed to black!
	save("advanced.status", null)
	save("advanced.bgcolor", null)
	wait(1)

	showDynamically("Now it's better.")
	wait(3)

	showDynamically("Can you wait 10 seconds?")
	showButton("Of course")

	waitWithCount(10)

	showDynamically("Now let's wait with a countdown!")
	wait(4)

	waitWithCountDown(10)

	// Let's now pick one toy randomly. We don't have to check if the keys are true,
	// the key picked is selected only from the values storing true
	showDynamically("And now let's pick one spanking implement randomly.")
	wait(5)
	def implement = getRandomItem(["toys.crop", "toys.paddle", "toys.whip"])
	if (implement == "toys.crop") implement = "crop"
	else if (implement == "toys.paddle") implement = "paddle"
	else if (implement == "toys.whip") implement = "whip"

	if (implement == null) showDynamically("You don't have any spanking toy!")
	else showDynamically("I picked a "+implement)
	wait(4)

	showDynamically("And that's all!! I hope you liked this example.\n\nSee you!!")
	wait(5)

	textSpeed = 20
	unshowDynamically("And that's all!! I hope you liked this example.\n\nSee you!!")
	setImageOffFancy("default-female.jpg")
	wait(1)
}

// This script is protected by this license: 
// CC by-NC, see http://creativecommons.org/licenses/by-nc/3.0/
